/* JS Option String Object */
function StrOpt(optstr) {
    if (!optstr) optstr = "";
    var argstrs = optstr.split(';');
    for (var i = 0; i < argstrs.length; ++i) {
        var pos = argstrs[i].search('=');
        if (pos != -1) {
            var name = argstrs[i].substring(0, pos);
            var value = argstrs[i].substr(pos+1);
            this[name] = value;
        }
    }
}

StrOpt.prototype.toOptString = function() {
    function isFunction(obj) {
        var getType = {};
        return obj && getType.toString.call(obj) == '[object Function]';
    }
    var str = "";
    for (var name in this) {
        if (isFunction(this[name])) continue;
        str += name + "=" + this[name] + ";";
    }
    return str;
};

var ydgwUpdater = {
  init: function() {
    window.removeEventListener("load", ydgwUpdater.init, true);
    ydgwUpdater.checkUpdate();
    ydgwUpdater.sendHeartBeat();
    YDGouwuExt.init();
  },

  installButton: function() {
    if (!document.getElementById('ydgw-toolbar-button')) {
      var toolbar = document.getElementById('nav-bar');
      var elem = document.getElementById('urlbar');
      var before = elem.nextElementSibling;
      toolbar.insertItem('ydgw-toolbar-button', before);
      toolbar.setAttribute("currentset", toolbar.currentSet);
      document.persist(toolbar.id, "currentset");
    }
  },

  getContents: function(aUrl) {
    var ioService = Components.classes["@mozilla.org/network/io-service;1"].
      getService(Components.interfaces.nsIIOService);
    var scriptableStream = Components.classes["@mozilla.org/scriptableinputstream;1"].
      getService(Components.interfaces.nsIScriptableInputStream);
    var unicodeConverter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].
      createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
    unicodeConverter.charset = "UTF-8";

    var channel = ioService.newChannel(aUrl, "UTF-8", null);
    var input = channel.open();
    scriptableStream.init(input);
    var str = scriptableStream.read(input.available());
    scriptableStream.close();
    input.close();

    try {
      return unicodeConverter.ConvertToUnicode(str);
    } catch(e) {
      return str;
    }
  },

  sendLog: function(type, action) {
    var logUrl = 'http://zhushou.huihui.cn/log?';
    if (type == 'INSTALL') {
      logUrl += 'type=ARMANI_EXTENSION_INSTALL';
    } else if (type == 'ACTION') {
      logUrl += 'type=ARMANI_EXTENSION_ACTION' + '&action=' + action;
    } else if (type == 'UNINSTALL') {
      logUrl += 'type=ARMANI_EXTENSION_UNINSTALL' + '&action=' + action;
    }
    logUrl += '&version=' + this.version + '&vendor=' + this.vendor; +
              '&browser=firefox';
    var xhr = new XMLHttpRequest();
    xhr.open('POST', logUrl, false);
    xhr.send(null);
  },

  checkUpdate: function() {
    var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                getService(Components.interfaces.nsIPrefService);
    prefs = prefs.getBranch("extensions.youdaogouwu.");

        // handle the vendor infomation
    var vendor_file = ydgwUpdater.getContents('chrome://youdaogouwu/content/vendor.data');
    var vendor = /[^\s]+/.exec(vendor_file)[0];
    if (vendor !== "youdao" || !prefs.prefHasUserValue("vendor") || 
        prefs.getCharPref('vendor').length !== 0 ) {
      prefs.setCharPref("vendor", vendor);
    }

    // previous version 
    var prevVersion = null;
    if (prefs.prefHasUserValue("currentversion")) {
      prevVersion = prefs.getCharPref("currentversion");
    }
    // just put the new version here;
    currVersion = "2.9";

    if (prevVersion !== currVersion) {
      prefs.setCharPref("currentversion", currVersion);
      vendor = prefs.getCharPref("vendor");
      ydgwUpdater.installButton();
      var url;
      var urlopt = "browser=firefox" + "&version=" + currVersion + "&vendor=" + vendor;
      var opt = new StrOpt(prefs.getCharPref('jsoptions'));
      if (!prevVersion || prevVersion === '0') { // new install
        opt['state'] = 'install';
        url = "http://zhushou.huihui.cn/installed?" + urlopt;
      } else { // update
        opt['state'] = 'update';
        url = "http://zhushou.huihui.cn/updatelog?" + urlopt;
      }
      prefs.setCharPref('jsoptions', opt.toOptString())
      window.setTimeout(function() { 
        gBrowser.selectedTab = gBrowser.addTab(url);
      }, 2000);
    }
  },

  sendHeartBeat: function() {
    var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                getService(Components.interfaces.nsIPrefService);
    prefs = prefs.getBranch("extensions.youdaogouwu.");

    var today = (new Date).getDate();
    if (prefs.prefHasUserValue('lastheartbeat')) {
      var last = prefs.getCharPref('lastheartbeat');
      if (last && last == today) return;
    }

    var guid = '';
    if (prefs.prefHasUserValue('guid'))
      guid = prefs.getCharPref('guid');
    if (guid === '') {
      for (var i = 1; i <= 32; ++i) {
        var n = Math.floor(Math.random()*16.0).toString(16);
        guid += n;
        if((i==8)||(i==12)||(i==16)||(i==20))
          guid += "-";
      }
      prefs.setCharPref('guid', guid);
    }
    var vendor = prefs.getCharPref("vendor");
    var version = prefs.getCharPref('currentversion');
    var url = 'http://zhushou.youdao.com/log?type=ARMANI_EXTENSION_HEARTBEAT' + 
      '&browser=firefox&extensionid=' + guid + 
      '&vendor=' + this.vendor + '&version=' + this.version;
    var img = new Image();
    img.src = url;
    
    prefs.setCharPref('lastheartbeat', today);
  }
};

window.addEventListener("load", ydgwUpdater.init, true);

